@@ -1,6 +1,8 @@
 """
 String to type annotation map to replace overriden botocore shapes.
 """
+from collections.abc import Iterable
+
 from mypy_boto3_builder.import_helpers.import_string import ImportString
 from mypy_boto3_builder.service_name import ServiceName, ServiceNameCatalog
 from mypy_boto3_builder.type_annotations.external_import import ExternalImport
@@ -10,6 +12,26 @@ from mypy_boto3_builder.type_annotations.type_subscript import TypeSubscript
 from mypy_boto3_builder.type_annotations.type_typed_dict import TypedDictAttribute, TypeTypedDict
 
 # FIXME: a hack to avoid cicular TypedDict in dynamodb package
+TableAttributeValueType: TypeSubscript = TypeSubscript(
+    Type.Union,
+    [
+        Type.bytes,
+        Type.bytearray,
+        Type.str,
+        Type.int,
+        Type.Decimal,
+        Type.bool,
+        TypeSubscript(Type.Set, [Type.int]),
+        TypeSubscript(Type.Set, [Type.Decimal]),
+        TypeSubscript(Type.Set, [Type.str]),
+        TypeSubscript(Type.Set, [Type.bytes]),
+        TypeSubscript(Type.Set, [Type.bytearray]),
+        Type.SequenceAny,
+        Type.MappingStrAny,
+        Type.none,
+    ],
+)
+
 AttributeValueTypeDef: TypeTypedDict = TypeTypedDict(
     "AttributeValueTypeDef",
     [
@@ -39,8 +61,9 @@ InvocationResponseTypeDef: TypeTypedDict = TypeTypedDict(
 )
 
 StreamingBodyType = ExternalImport(ImportString("botocore", "response"), "StreamingBody")
+ShapeTypeMap = dict[ServiceName, dict[str, FakeAnnotation]]
 
-SHAPE_TYPE_MAP: dict[ServiceName, dict[str, FakeAnnotation]] = {
+SHAPE_TYPE_MAP: ShapeTypeMap = {
     ServiceNameCatalog.all: {
         "integer": Type.int,
         "long": Type.int,
@@ -58,10 +81,11 @@ SHAPE_TYPE_MAP: dict[ServiceName, dict[str, FakeAnnotation]] = {
     },
     ServiceNameCatalog.dynamodb: {
         "AttributeValueTypeDef": AttributeValueTypeDef,
+        "AttributeValueTableTypeDef": TableAttributeValueType,
     },
 }
 
-OUTPUT_SHAPE_TYPE_MAP: dict[ServiceName, dict[str, FakeAnnotation]] = {
+OUTPUT_SHAPE_TYPE_MAP: ShapeTypeMap = {
     ServiceNameCatalog.all: {
         "timestamp": Type.datetime,
         "blob": Type.bytes,
@@ -70,51 +94,27 @@ OUTPUT_SHAPE_TYPE_MAP: dict[ServiceName, dict[str, FakeAnnotation]] = {
 }
 
 
-def get_shape_type_stub(service_name: ServiceName, shape_name: str) -> FakeAnnotation | None:
+def get_shape_type_stub(
+    shape_type_maps: Iterable[ShapeTypeMap],
+    service_name: ServiceName,
+    shape_name: str,
+) -> FakeAnnotation | None:
     """
     Get stub type for input botocore shape.
 
     Arguments:
-        service_name -- Service name.
-        shape_name -- Target TypedDict name.
+        shape_type_map -- Map to lookup
+        service_name -- Service name
+        shape_name -- Target TypedDict name
 
     Returns:
         Type annotation or None.
     """
-    if service_name not in SHAPE_TYPE_MAP:
-        if service_name == ServiceNameCatalog.all:
-            return None
-        return get_shape_type_stub(ServiceNameCatalog.all, shape_name)
+    service_names = (service_name, ServiceNameCatalog.all)
+    for shape_type_map in shape_type_maps:
+        for current_service_name in service_names:
+            service_shape_type_map = shape_type_map.get(current_service_name, {})
+            if shape_name in service_shape_type_map:
+                return service_shape_type_map[shape_name]
 
-    service_shape_type_map = SHAPE_TYPE_MAP[service_name]
-    if shape_name not in service_shape_type_map:
-        if service_name == ServiceNameCatalog.all:
-            return None
-        return get_shape_type_stub(ServiceNameCatalog.all, shape_name)
-
-    return service_shape_type_map[shape_name]
-
-
-def get_output_shape_type_stub(service_name: ServiceName, shape_name: str) -> FakeAnnotation | None:
-    """
-    Get stub type for output botocore shape.
-
-    Arguments:
-        service_name -- Service name.
-        shape_name -- Target TypedDict name.
-
-    Returns:
-        Type annotation or None.
-    """
-    if service_name not in OUTPUT_SHAPE_TYPE_MAP:
-        if service_name == ServiceNameCatalog.all:
-            return None
-        return get_output_shape_type_stub(ServiceNameCatalog.all, shape_name)
-
-    service_shape_type_map = OUTPUT_SHAPE_TYPE_MAP[service_name]
-    if shape_name not in service_shape_type_map:
-        if service_name == ServiceNameCatalog.all:
-            return None
-        return get_output_shape_type_stub(ServiceNameCatalog.all, shape_name)
-
-    return service_shape_type_map[shape_name]
+    return None
